package com.ahs.flutter.ahs_flutter_router

import android.app.Activity
import android.content.Context
import android.content.Intent
import android.net.Uri
import androidx.annotation.NonNull
import com.ahs.flutter.ahs_flutter_router.channel.FlutterBinaryMessenger
import com.ahs.flutter.ahs_flutter_router.intent.FlutterModuleIntentManager

import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import java.io.Serializable

/** AhsFlutterRouterPlugin */
class AhsFlutterRouterPlugin : FlutterPlugin, MethodCallHandler, ActivityAware {
    private lateinit var channel: MethodChannel
    private lateinit var context: Context
    private lateinit var activity: Activity

    override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        FlutterBinaryMessenger.setBinaryMessenger(flutterPluginBinding.binaryMessenger)
        channel =
            MethodChannel(flutterPluginBinding.binaryMessenger, AhsRouterConstant.routerChannelName)
        channel.setMethodCallHandler(this)
    }

    override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
        when (call.method) {
            // Flutter传递参数到Native端的方法
            AhsRouterConstant.flutterToNative -> {
                var param = call.arguments as Map<String, Any>
                getFlutterParam(param)
                result.success(null)
            }
            AhsRouterConstant.backNativePage -> {
                val backIntent = Intent()
                backIntent.putExtra("message", call.argument<Any>("back") as String?)
                activity.setResult(Activity.RESULT_OK, backIntent)
                activity.finish()
                result.success("成功返回第一个原生页面")
            }
            else -> {
                result.notImplemented()
            }
        }
    }


    /**
     * 获取 Flutter 端传递过来的参数进行跳转界面
     */
    private fun getFlutterParam(param: Map<String, Any>) {
        val intent =
            Intent(activity, FlutterModuleIntentManager.mModuleIntentBuilder.getActivityClz())
        intent.putExtra(AhsRouterConstant.flutterToNative, param as Serializable)
        activity.startActivity(intent)
    }

    override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
        channel.setMethodCallHandler(null)
    }

    override fun onAttachedToActivity(binding: ActivityPluginBinding) {
        activity = binding.activity
    }

    override fun onDetachedFromActivityForConfigChanges() {
    }

    override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {
    }

    override fun onDetachedFromActivity() {
    }
}
