import 'dart:async';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:provider/provider.dart';

import 'ahs_flutter_router.dart';

//统一路由跳转管理类
class AhsNavigator extends ChangeNotifier {
  AhsNavigator._();

  static final _instance = AhsNavigator._();

  factory AhsNavigator.getInstance() => _instance;

  static AhsNavigator of(BuildContext context) {
    return Provider.of<AhsNavigator>(context, listen: false);
  }

  final List<Page> _pages = [];

  List<Page> get pages => List.unmodifiable(_pages);

  final _navigatorKey = GlobalKey<NavigatorState>();

  GlobalKey<NavigatorState> get navigatorKey => _navigatorKey;

  //初始化根界面
  void initRootPage(Widget widget, String pageName) {
    _pages.clear();
    _pages.add(MaterialPage(
      child: widget,
      name: pageName,
      key: Key(pageName),
    ));
    notifyListeners();
  }

  //提供跳转界面的方法
  void pushPage(Widget widget, String pageName) {
    _pages.add(MaterialPage(
      child: widget,
      name: pageName,
      key: Key(pageName),
    ));
    notifyListeners();
  }

  void returnWith(bool value) {
    if (_boolResultCompleter != null) {
      _pages.removeLast();
      _boolResultCompleter.complete(value);
      notifyListeners();
    }
  }

  void addOtherPageBeneath(Widget child, String pageName) {
    final inserted = MaterialPage(
      child: child,
      key: Key(pageName),
      name: pageName,
    );
    _pages.insert(_pages.length - 1, inserted);
    notifyListeners();
  }

  bool isRootPage(Key key) {
    final value = _pages
        .elementAt(0)
        .key == key;

    return value;
  }

  void makeRootPage() {
    _pages.removeRange(0, _pages.length - 1);
    notifyListeners();
  }

  void replaceTopWith(Widget child) {
    _pages.removeAt(_pages.length - 1);
    _pages.add(
      MaterialPage(
        child: child,
        key: UniqueKey(),
        name: '${child.runtimeType}',
      ),
    );
    notifyListeners();
  }

  Completer<bool> _boolResultCompleter;

  void _setResult(dynamic result) {
    if (result is bool && _boolResultCompleter != null) {
      _boolResultCompleter.complete(result);
    }
    if (result == null) {
      print('Result was null');
    }
  }

  Future<bool> waitForResult(Widget widget, String pageName) async {
    _boolResultCompleter = Completer<bool>();
    _pages.add(
      ResultablePage(
        child: widget,
        key: Key(pageName),
        name: pageName,
      ),
    );
    notifyListeners();
    return _boolResultCompleter.future;
  }

  void pop() {
    if (_pages.length>1) {
      _pages.removeLast();
      notifyListeners();
    }else{
      //返回到原生界面
      returnLastNativePage(Map());
    }
  }

  void didPop(Page page, dynamic result) {
    _pages.remove(page);
    if (page is ResultablePage) {
      _setResult(result);
    }
    notifyListeners();
  }

  @override
  void dispose() {
    super.dispose();
    _boolResultCompleter?.complete();
  }

  
  ////跳转到native 相关
  void toNative(Map<String, dynamic> param) {
    AhsFlutterRouter.toNativeByParam(param);
  }

  /**/
  void returnLastNativePage(Map<String, dynamic> param) {
    AhsFlutterRouter.returnLastNativePage(param);
  }
}

class ResultablePage extends MaterialPage {
  const ResultablePage({
    @required Widget child,
    LocalKey key,
    String name,
  }) : super(
    key: key,
    name: name,
    child: child,
  );
}
